@@ -1,11 +1,11 @@
 use super::PagedWriter;
-use crate::{error::Error, io::ManagedFile, ArcBytes};
+use crate::{error::Error, ArcBytes};
 
 pub trait BinarySerialization: Send + Sync + Sized {
-    fn serialize_to<File: ManagedFile>(
+    fn serialize_to(
         &mut self,
         writer: &mut Vec<u8>,
-        paged_writer: &mut PagedWriter<'_, File>,
+        paged_writer: &mut PagedWriter<'_>,
     ) -> Result<usize, Error>;
     fn deserialize_from(
         reader: &mut ArcBytes<'_>,
@@ -14,10 +14,10 @@ pub trait BinarySerialization: Send + Sync + Sized {
 }
 
 impl BinarySerialization for () {
-    fn serialize_to<File: ManagedFile>(
+    fn serialize_to(
         &mut self,
         _writer: &mut Vec<u8>,
-        _paged_writer: &mut PagedWriter<'_, File>,
+        _paged_writer: &mut PagedWriter<'_>,
     ) -> Result<usize, Error> {
         Ok(0)
     }
